#!/bin/bash
# Copyright (c) 2025 Huawei Technologies Co., Ltd. All rights reserved.
set -e

# Function to display the usage of the script
display_usage() {
    echo "Usage: $0 [-h] [-i install_path] package_file1 package_file2 ... package_fileN"
    echo "Options:"
    echo "  -h                 Display this help message and exit."
    echo "  -i install_path    Specify the installation path."
}

# Function to check if the package exists in the current working folder
validate_packages() {
    local packages=("$@")
    for package in "${packages[@]}"; do
        if [ ! -f "$package" ]; then
            echo "Error: Package $package does not exist in the current working folder."
            exit 1
        fi
    done
}

# Function to check if the host already has a package containing "exagear" installed
check_exagear_installed() {
    yum list 2>/dev/null | grep -q "exagear" && return 0 || return 1
}

# Function to install packages using rpm
install_packages() {
    local packages=("$@")
    for package in "${packages[@]}"; do
        sudo rpm -U --oldpackage --force "$package"
    done
}

# Function to check if a given path is a absolute path
check_absolute_path() {
    case "$1" in
        /*) return 0 ;;
        *) return 1 ;;
    esac
}

# Function to mount install path to /opt/exagear
umount_opt() {
    if mountpoint -q /opt/exagear 2>/dev/null; then
        sudo umount /opt/exagear
    elif [ -d /opt/exagear ]; then
        echo "/opt/exagear is a directory. Please uninstall or use rpm install." && exit 1
    else
        sudo mkdir -p /opt/exagear
    fi
}

# Main script logic
main() {
    if systemctl is-active --quiet exagear-x86_32-opt.service exagear-x86_64-opt.service; then
        sudo systemctl stop exagear-x86_32-opt.service exagear-x86_64-opt.service
    fi
    local install_path=""
    local packages=()

    # Parse arguments
    while [ $# -gt 0 ]; do
        case "$1" in
            -h) display_usage; exit 0 ;;
            -i) install_path="$2"; shift 2 ;;
            *) packages+=("$1"); shift ;;
        esac
    done

    # Validate packages
    validate_packages "${packages[@]}"

    # Check if install_path is a full folder path
    if [ -n "$install_path" ]; then
        if ! check_absolute_path "$install_path"; then
            echo "Error: Install path $install_path is not a valid directory." && exit 1
        fi
    fi

    # Check if ExaGear is installed
    if ! check_exagear_installed ; then
        if [ -z "$install_path" ]; then
            install_packages "${packages[@]}"
        else
            if [ "$install_path" != "/opt/exagear" ]; then
                umount_opt
                mkdir -p "$install_path"
                sudo mount --bind "$install_path" /opt/exagear
            fi
            install_packages "${packages[@]}"
        fi
    else
        if [ -z "$install_path" ]; then
            install_packages "${packages[@]}"
        else
            mkdir -p "$install_path"
            if [ "$install_path" != "/opt/exagear" ]; then
                umount_opt
                sudo mount --bind "$install_path" /opt/exagear
            fi
            install_packages "${packages[@]}"
        fi
    fi

    cp -r /etc/exagear-guest-for-centos-7-x86_64.conf /etc/exagear.conf.d /usr/bin/exagear /usr/bin/exagear-manage $install_path
    sudo systemctl restart exagear-x86_32-opt.service exagear-x86_64-opt.service
}

# Call the main function with all script arguments
main "$@"
