#!/bin/bash
# Copyright Huawei Technologies Co., Ltd. 2019-2024. All rights reserved.
# Note: Do not run this file alone

set -e
DISTR_NAME=${distr_name}
DISTR_ARCH="x86_64"
EXAGEAR_VERSION=${version}
IMAGE_DIR_NAME="${DISTR_NAME}-${DISTR_ARCH}"
CONFDIR="${SOURCE_DIR}/configs"
DISTR_ARCH_2ND=$(cat ${SOURCE_DIR}/configs/2nd_arch)

function gen_rpm_spec_file() {
cat << EOF
Summary: ExaGear binary translation system - guest image ${DISTR_NAME}-${DISTR_ARCH}
Name: exagear-guest-for-${DISTR_NAME}-${DISTR_ARCH}
Group: Emulators
Version: ${EXAGEAR_VERSION}
Release: 1
License: Proprietary
Provides: exagear-guest
Requires: tar exagear-utils
Autoreq: 0

%description
Guest image (${DISTR_NAME} ${DISTR_ARCH}) for ExaGear

%post
function list_native_services() {
    pushd /opt/exagear/images/${IMAGE_DIR_NAME} >/dev/null
    ALL_SYSTEMD_DIRS="lib/systemd/system etc/systemd/system/"

    if [ "\`readlink lib\`" != "usr/lib" ]; then
        ALL_SYSTEMD_DIRS="\${ALL_SYSTEMD_DIRS} usr/lib/systemd/system/"
    fi

    find \${ALL_SYSTEMD_DIRS} -maxdepth 1 -type f 2> /dev/null

    popd >/dev/null
}

if [ ! -e "/opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/unpacked" ]
then
	echo "Unpacking archive with guest image: /opt/exagear/images/${IMAGE_DIR_NAME}.tar.gz ..."
	tar -xzf /opt/exagear/images/${IMAGE_DIR_NAME}.tar.gz -C /opt/exagear/images/${IMAGE_DIR_NAME}
	echo ${EXAGEAR_VERSION} > /opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/unpacked
	echo "...done."
	list_native_services > /opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/native-systemd-services
else
	echo "Found unpacked exagear image in /opt/exagear/images/${IMAGE_DIR_NAME}. Package will be updated, but guest image content will be unchanged."
fi

ln -sf /etc/exagear-guest-for-${DISTR_NAME}-${DISTR_ARCH}.conf /etc/exagear-guest-${DISTR_ARCH}.conf
if [ ! -z "${DISTR_ARCH_2ND}" ]
then
        ln -sf /etc/exagear-guest-for-${DISTR_NAME}-${DISTR_ARCH}.conf /etc/exagear-guest-${DISTR_ARCH_2ND}.conf
fi
ln -sf /etc/exagear-guest-for-${DISTR_NAME}-${DISTR_ARCH}.conf /etc/exagear-guest.conf
ln -sf /opt/exagear/integration/systemctl /opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/systemctl

IMAGE_DIR_NAME=${IMAGE_DIR_NAME} /opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/create-vpaths-list-stubs.sh

systemctl daemon-reload
exit 0

%preun
if [ \$1 -eq 0 ] # uninstall, not upgrade
then
    if [ -x /opt/exagear/integration/stop-all-guest-systemd-units.sh ]
    then
        . /etc/exagear-guest-for-${DISTR_NAME}-${DISTR_ARCH}.conf
        . /opt/exagear/integration/stop-all-guest-systemd-units.sh
    fi
fi

%postun

if [ \$1 -eq 0 ] # uninstall, not upgrade
then
    [ -L "/opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/systemctl" ] && unlink /opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/systemctl
    [ -L "/etc/exagear-guest.conf" ] && unlink /etc/exagear-guest.conf
    [ -L "/etc/exagear-guest-${DISTR_ARCH}.conf" ] && unlink /etc/exagear-guest-${DISTR_ARCH}.conf
    [ -L "/etc/exagear-guest-${DISTR_ARCH_2ND}.conf" ] && unlink /etc/exagear-guest-${DISTR_ARCH_2ND}.conf
    rm -rf /opt/exagear/images/${DISTR_NAME}-${DISTR_ARCH}
fi

systemctl daemon-reload
exit 0

%files
%defattr(-,root,root)
/opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/vpaths-list
/opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/opaths-list
/opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/utmp-list
%attr(755, root, root) /opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/create-vpaths-list-stubs.sh

/opt/exagear/images/${IMAGE_DIR_NAME}.tar.gz
/etc/exagear-guest-for-${DISTR_NAME}-${DISTR_ARCH}.conf
%dir /opt/exagear/images/${IMAGE_DIR_NAME}/
EOF
}

function gen_exagear_guest_conf_file() {
cat << EOF
EXAGEAR_GUEST_IMAGE_NAME=${DISTR_NAME}-${DISTR_ARCH}
EXAGEAR_GUEST_WRAP_MARKER="__WRAPPED_FOR_EXAGEAR_${DISTR_NAME}-${DISTR_ARCH}"
EXAGEAR_IMAGE_NAME=${DISTR_NAME}-${DISTR_ARCH}
EXAGEAR_IMAGE_ARCH=${DISTR_ARCH}
$([ -e "${CONFDIR}/config" ] && cat ${CONFDIR}/config)
EOF
}

function guest_image_build() {
    local _path_to_templ=$(realpath ${1})
    : ${TMPDIR:=/tmp}
    trap '[ -n "${BUILD_ROOT}" ] && rm -rf "${BUILD_ROOT}"' EXIT
    BUILD_ROOT=""
    mask_backup=$(umask)
    umask 077
    BUILD_ROOT=$(mktemp -d "${TMPDIR}/tpm.XXXXXXXXXXXXX")
    if [ $? -ne 0 ]; then
        echo "Make temp. BUILD_ROOT directory failed!"
        umask "${mask_backup}"
        exit 1
    fi
    umask "${mask_backup}"

    mkdir ${BUILD_ROOT}/BUILD
    mkdir ${BUILD_ROOT}/SPECS
    mkdir ${BUILD_ROOT}/RPMS
    mkdir ${BUILD_ROOT}/install
    mkdir -p  ${BUILD_ROOT}/install/etc/
    mkdir -p  ${BUILD_ROOT}/install/opt/exagear/images/

    gen_rpm_spec_file > ${BUILD_ROOT}/SPECS/exagear-guest-for-${DISTR_NAME}-${DISTR_ARCH}.spec
    gen_exagear_guest_conf_file >  ${BUILD_ROOT}/install/etc/exagear-guest-for-${DISTR_NAME}-${DISTR_ARCH}.conf

    cp -L ${_path_to_templ} ${BUILD_ROOT}/install/opt/exagear/images/${IMAGE_DIR_NAME}.tar.gz

    install -D -m 644 ${SOURCE_DIR}/configs/vpaths-list ${BUILD_ROOT}/install/opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/vpaths-list
    install -D -m 644 ${SOURCE_DIR}/configs/utmp-list ${BUILD_ROOT}/install/opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/utmp-list
    install -D -m 644 ${SOURCE_DIR}/configs/opaths-list ${BUILD_ROOT}/install/opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/opaths-list
    install -D -m 755 ${SOURCE_DIR}/scripts/create-vpaths-list-stubs.sh ${BUILD_ROOT}/install/opt/exagear/images/${IMAGE_DIR_NAME}/.exagear/create-vpaths-list-stubs.sh

    pushd ${BUILD_ROOT}/SPECS
    rpmbuild -bb --define "_allow_root_build yes" \
                 --define "_topdir ${BUILD_ROOT}" \
                 --define "_tmppath ${BUILD_ROOT}/tmp" \
                 --define "buildroot ${BUILD_ROOT}/install" \
                 --target noarch \
                 exagear-guest-for-${DISTR_NAME}-${DISTR_ARCH}.spec

    BUILT_PACKAGE=$(realpath $(find ${BUILD_ROOT}/RPMS -type f))
    echo $(basename ${BUILT_PACKAGE})
    popd
    cp ${BUILT_PACKAGE} ${ORGN_DIR}
}
