#!/bin/bash
# Copyright Huawei Technologies Co., Ltd. 2024. All rights reserved.

# Usage: Run with the following arg.s on amd64_ubuntu-20.04 under root account:
#   -v, --version               ExaGear version, i.e. version of the other ExaGear installation packages
#   -d, --distr-name            OS distribution, e.g. centos-7, centos-8
#   -i, --iso-path              Where to find the OS iso image
#   -m, --mod-libc              with patched lib.s or not, valid value yes, no
#
# e.g. ./buildGuestImageCentOS.sh -v 3081 -d centos-8 -i ~/CentOS-8.5.2111-x86_64-dvd1.iso -m no"
#
# Usage: Run with the following arg.s on amd64_ubuntu-20.04 under root account:
#   -v, --version               ExaGear version, refer the version of other ExaGear packages
#   -d, --distr-name            OS distribution, e.g. centos-7, centos-8
#   -i, --iso-path              Where to find the OS iso image
#   -m, --mod-libs              with patched libc or not, valid value yes, no
#   -u, --url-repos             accessable yum-repo server url
#   -p, --patches-dir           customized libc patches location
#
# e.g. ./buildGuestImageCentOS.sh -v 3081 -d centos-8 -i ~/CentOS-8.5.2111-x86_64-dvd1.iso -m yes -u <yum/repo/url> -p ~/patches"

set -e
ORGN_DIR="$(pwd)"
SOURCE_DIR="$(cd $(dirname ${0});pwd)"
ename="${0}"

function usage_prompt_info() {
    local _ename="${1}"
    local _is_patched="${2}"

    if [ "${_is_patched}" == "no" ]; then
        sed -n "4,10p" ${_ename}
    else
        sed -n "12,20p" ${_ename}
    fi
}

function iso_shasum_check() {
    local _distr_name="${1}"
    local _iso_path="${2}"

    if [ "${_distr_name}" == "centos-8" ]; then
        local _iso_name="CentOS-8.5.2111-x86_64-dvd1.iso"
        local _offical_sha256="3b795863001461d4f670b0dedd02d25296b6d64683faceb8f2b60c53ac5ebb3e"
        local _iso_sha256=$(shasum -a 256 ${_iso_path} | awk -F ' ' '{print $1}')
    fi

    if [ "${_offical_sha256}" != "${_iso_sha256}" ]; then
        trans_protcl="https"
        offical_vault_serv="vault.centos.org"
        huawei_vault_serv="mirrors.huaweicloud.com"
        echo -e "Bad iso image, download it from offical-site and rightly:
            from: ${trans_protcl}://${offical_vault_serv}/ or ${trans_protcl}://${huawei_vault_serv}/centos-vault/
            name: ${_iso_name}
          sha256: ${_offical_sha256}"
        exit 1
    fi
}

function local_yum_repo_gen() {
    local _mnt_point="/mnt/x86/${distr_name}/"

    umount ${_mnt_point} || true
    [ -d "${_mnt_point}" ] && rm -rf ${_mnt_point}
    mkdir -p ${_mnt_point}
    mount -o loop ${iso_path} ${_mnt_point}
    cat << EOF > ${1}
[Base]
name=${distr_name}_Base_Repo
baseurl=file://${_mnt_point}/BaseOS/
gpgcheck=0

[AppStream]
name=${distr_name}_AppStream_Repo
baseurl=file://${_mnt_point}/AppStream/
gpgcheck=0

EOF
}

function guest_rootfs_customize() {
    local _rootfs="${1}"
    local _os_releasever="${2}"

    pushd ${_rootfs} >/dev/null
    [ -n "$(ls -A ./var/cache/yum/)" ] && rm -rf ./var/cache/yum/* || true
    [ -n "$(ls -A ./var/tmp/)" ] && rm -rf ./var/tmp/*
    [ -n "$(ls -A ./tmp/)" ] && rm -rf ./tmp/*

    mkdir -p ./etc/dnf/vars
    echo -e "${_os_releasever}" > ./etc/dnf/vars/releasever
    mkdir -p ./etc/ldap && mkdir -p ./etc/resolvconf
    mkdir -p ./var/lib/dbus && mkdir -p ./usr/share/X11
    mkdir -p ./etc/resolfconv && mkdir -p ./usr/share/icons
    touch ./bin/systemctl && chmod +x ./bin/systemctl

    if [ "${mod_libs}" == "yes" ]; then
        chown root:root ${3}/*
        cp -f ${3}/libgcc_s.so.1 $(realpath ./usr/lib64/libgcc_s-8*.so.1)
        cp -f ${3}/libgcc_s.so.1 $(realpath ./lib64/libgcc_s-8*.so.1)
        cp -f ${3}/ld.so ./lib64/ld-2.28.so
        cp -f ${3}/libdl.so ./lib64/libdl-2.28.so
        cp -f ${3}/libc.so ./lib64/libc-2.28.so
    fi
    popd >/dev/null
}

# read named arguments
required=0
while [[ $# -gt 0 ]];
do
  arg="$1"
  case "$arg" in
    -h|--help)
      usage_prompt_info ${ename} "no"
      echo -e "\nOR, WITH LIBS PATCHED: "
      usage_prompt_info ${ename} "yes"
      exit 0
    ;;
    -v|--version)
      version=${2}
      required=$(($required+1))
    ;;
    -d|--distr-name)
      distr_name=${2}
      required=$(($required+1))
    ;;
    -i|--iso-path)
      iso_path=${2}
      required=$(($required+1))
    ;;
    -m|--mod-libs)
      mod_libs=${2}
      required=$(($required+1))
    ;;
    -u|--url-repos)
      url_repos=${2}
      required=$(($required+1))
    ;;
    -p|--patches-dir)
      patches_dir=${2}
      required=$(($required+1))
    ;;
    *)
      echo "Unknown option ${arg}."
    ;;
  esac
  if [[ $# -le 1 ]]; then
    echo "Value of argument ${arg} not specified."
  fi
  shift 2
done

if [ $required -lt 4 ]; then
    usage_prompt_info ${ename} "others"
    exit 1
else
    if [ "${mod_libs}" == "yes" ]; then
        if [ $required -lt 6 ]; then
            usage_prompt_info ${ename} ${mod_libs}
            exit 1
        elif [ -z "${patches_dir}" ]; then
            echo "No patches provided for lib.s customization."
            exit 1
        fi
    fi
fi

WORKSHOP="build_workshop"
[ -d "${WORKSHOP}" ] && rm -rf ${WORKSHOP}
mkdir ${WORKSHOP}
WORKSHOP=$(realpath ${WORKSHOP})
pushd ${WORKSHOP} >/dev/null

### Libs customization
if [ "${mod_libs}" == "yes" ]; then
    function libs_patched_with_docker() {
        local _tmp_workshop="${1}"
        local _docker_image="${2}"

        cp -f ${SOURCE_DIR}/scripts/libs-build.sh ${_tmp_workshop}/.
        cp -rf ${SOURCE_DIR}/patches ${_tmp_workshop}/
        mkdir -p ${_tmp_workshop}/libs_repo
        local _patches_repo=$(realpath ${_tmp_workshop}/patches)
        local _libs_repo=$(realpath ${_tmp_workshop}/libs_repo)
        local _docker_container="docker-container-libs-build-${distr_name}"

        docker rm -f ${_docker_container} || true
        docker run -v /root:/root --name ${_docker_container} ${_docker_image} /bin/bash \
                   ${_tmp_workshop}/libs-build.sh ${_patches_repo} ${_libs_repo} ${url_repos}
        cp -rf ${_libs_repo}/* ${3}
        docker rm -f ${_docker_container} || true
    }

    _docker_image="amd64/centos:8"
    _dist_kits="/root/${distr_name}"
    _libs_repo="${WORKSHOP}/_libs_repo"
    [ -d "${_dist_kits}" ] && rm -rf ${_dist_kits}
    mkdir -p ${_libs_repo} && mkdir -p ${_dist_kits}
    libs_patched_with_docker ${_dist_kits} ${_docker_image} ${_libs_repo}
fi

## checksum for iso
iso_shasum_check ${distr_name} ${iso_path}

### Set-up local yum repo
_local_yum_repo_file="${WORKSHOP}/cfg.repo"
local_yum_repo_gen ${_local_yum_repo_file}

### Make the GuestOS rootfs and template-package
_os_releasever=$(echo ${distr_name} | awk -F '-' '{print $2}')
_rootfs="rootfs"
[ -d "${_rootfs}" ] && rm -rf ${_rootfs}
mkdir -p ${_rootfs}
_rootfs=$(realpath ${_rootfs})
_packages_install="bash bash-completion rootfiles vim-minimal sudo yum iputils initscripts tzdata"
yum -y -c ${_local_yum_repo_file} --releasever=${_os_releasever} --disablerepo="*" --enablerepo=Base,AppStream --installroot=${_rootfs} install ${_packages_install} || true
guest_rootfs_customize ${_rootfs} ${_os_releasever} ${_libs_repo}
pushd ${_rootfs} >/dev/null
tar -czf ${WORKSHOP}/template-${distr_name}_amd64.tar.gz ./*
popd >/dev/null

### ExaGear guest-image build
source ${SOURCE_DIR}/scripts/guest-image-rpm-build.sh
guest_image_build "${WORKSHOP}/template-${distr_name}_amd64.tar.gz"
popd >/dev/null

[ -d "${WORKSHOP}" ] && rm -rf ${WORKSHOP}
umount ${_mnt_point} || true
[ -d "${_mnt_point}" ] && rm -rf ${_mnt_point}
echo "Built Successfully!!!"
