#!/usr/bin/env python3

"""
Script used for annotation of dynamically generated UBT code (traces and regions) when run under `perf report` tool.
At the moment it works only when host architecture is aarch64 or x86_64.
"""

import sys
import os
import re
import struct
import argparse
import subprocess
from collections import namedtuple


# Dump commandline, because perf report doesn't show stderr in case of errors in this script
print("Running following command (if output is empty, run it manually)")
print(*sys.argv, sep=" ", flush=True)

DEFAULT_ARCH = subprocess.check_output(["uname", "-m"], encoding="ascii").strip()

parser = argparse.ArgumentParser()
parser.add_argument("-m", "--architecture", default=DEFAULT_ARCH)
parser.add_argument("--start-address", required=True)
parser.add_argument("--stop-address", required=True)
parser.add_argument("path")

Section = namedtuple("Section", ["vaddr_start", "size", "file_offset"])


# Assuming little-endian!
def read_codefile_sections(path):
    sections = []
    with open(path, "rb") as f:
        magic = f.read(8)
        if magic != b"UBTCODE.":
            # Parse old version of .code file (approx pre trunk r2070-r2100)
            if magic != struct.pack("<Q", 0x800818c00000): # x64a64 default gcodebuf start
                print("Warning: unexpected header")
            [vaddr] = struct.unpack("<Q", magic)
            size = os.stat(path).st_size
            sections.append(Section(vaddr_start=vaddr, size=size, file_offset=8))
        else:
            # Recent version of .code file
            [sections_count] = struct.unpack("<Q", f.read(8))
            sections = []
            file_offset = 8 + 8 + 16 * sections_count
            for _ in range(sections_count):
                vaddr, size = struct.unpack("<QQ", f.read(16))
                sections.append(Section(vaddr_start=vaddr, size=size, file_offset=file_offset))
                file_offset += size
    return sections


def find_section_for(vaddr_start, vaddr_stop, sections):
    candidates = [
        sect for sect in sections
        if sect.vaddr_start <= vaddr_start and vaddr_stop <= sect.vaddr_start + sect.size
    ]
    if not candidates:
        raise ValueError("Adress range {:x} {:x} outsize of any section".format(vaddr_start, vaddr_stop))
    return candidates[0]


args, _ = parser.parse_known_args()

if args.architecture not in ("x86_64", "aarch64"):
    print("Unknown architecture", args.architecture)
    exit(1)

vaddr_start, vaddr_stop = int(args.start_address, base=16), int(args.stop_address, base=16)

match = re.match(r"perf-(\d+).map", os.path.basename(args.path))
if match is None:
    objdump_args = sys.argv[1:]
    os.execvp("objdump", ["objdump", *objdump_args])
    assert False
pid = int(match.groups()[0])
codepath = os.path.join(os.path.dirname(args.path), f"perf-{pid}.code")
assert os.path.exists(codepath)

sections = read_codefile_sections(codepath)
section = find_section_for(vaddr_start, vaddr_stop, sections)

# objdump -m switch doesn't expect the same thing as `uname -m` shows
objdump_arch = {
    "x86_64": "i386:x86-64",
    "aarch64": "aarch64",
}[args.architecture]

objdump_args = [
    "-b", "binary",
    "--no-show-raw",
    "-m", objdump_arch,
    "--disassemble-all",
    f"--adjust-vma={-section.file_offset + section.vaddr_start}",
    f"--start-address=0x{vaddr_start:x}",
    f"--stop-address=0x{vaddr_stop:x}",
    codepath,
]

file_start = section.file_offset + vaddr_start - section.vaddr_start
file_stop  = section.file_offset + vaddr_stop - section.vaddr_start
print(f"""
Translation, vaddr start = 0x{vaddr_start:x}, stop = 0x{vaddr_stop:x}
Inside codefile offset start = 0x{file_start:x}, stop = 0x{file_stop:x}
Map file path {args.path}, code file path {codepath}
Arch: {args.architecture}
""".strip(), flush=True) # flush is important, since execve forgets any internal buffers

os.execvp("objdump", ["objdump", *objdump_args])
assert False
